@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-from typing import Any, Tuple
+from typing import Any, List, Tuple
 
 from .abc import Abstract_ADBNX_Controller
 from .typings import Json, NxData, NxId
@@ -13,62 +13,61 @@ class ADBNX_Controller(Abstract_ADBNX_Controller):
     Responsible for controlling how nodes & edges are handled when
     transitioning from ArangoDB to NetworkX, and vice-versa.
 
-    You can derive your own custom ADBNX_Controller, but it is not
-    necessary for Homogeneous graphs.
+    You can derive your own custom ADBNX_Controller.
     """
 
-    def _prepare_arangodb_vertex(self, adb_vertex: Json, col: str) -> NxId:
+    def _prepare_arangodb_vertex(self, adb_vertex: Json, col: str) -> None:
         """Prepare an ArangoDB vertex before it gets inserted into the NetworkX
         graph.
 
         Given an ArangoDB vertex, you can modify it before it gets inserted
-        into the NetworkX graph, and/or derive a custom node id for networkx to use.
-        In most cases, it is only required to return the ArangoDB _id of the vertex.
+        into the NetworkX graph, and/or derive a custom node id for NetworkX
+        to use by updating the "_id" attribute of the vertex (otherwise the
+        vertex's current "_id" value will be used)
 
         :param adb_vertex: The ArangoDB vertex object to (optionally) modify.
         :type adb_vertex: adbnx_adapter.typings.Json
         :param col: The ArangoDB collection the vertex belongs to.
         :type col: str
-        :return: The ArangoDB _id attribute of the vertex.
-        :rtype: str
         """
-        adb_vertex_id: str = adb_vertex["_id"]
-        return adb_vertex_id
+        return
 
-    def _prepare_arangodb_edge(self, adb_edge: Json, col: str) -> NxId:
+    def _prepare_arangodb_edge(self, adb_edge: Json, col: str) -> None:
         """Prepare an ArangoDB edge before it gets inserted into the NetworkX
         graph.
 
         Given an ArangoDB edge, you can modify it before it gets inserted
-        into the NetworkX graph. In most cases, it is only required to
-        return the ArangoDB _id of the edge.
+        into the NetworkX graph.
 
         :param adb_edge: The ArangoDB edge object to (optionally) modify.
         :type adb_edge: adbnx_adapter.typings.Json
         :param col: The ArangoDB collection the edge belongs to.
         :type col: str
-        :return: The ArangoDB _id attribute of the edge.
-        :rtype: str
         """
-        adb_edge_id: str = adb_edge["_id"]
-        return adb_edge_id
+        return
 
-    def _identify_networkx_node(self, nx_node_id: NxId, nx_node: NxData) -> str:
-        """Given a NetworkX node, identify what ArangoDB collection it should
-        belong to.
+    def _identify_networkx_node(
+        self, nx_node_id: NxId, nx_node: NxData, adb_v_cols: List[str]
+    ) -> str:
+        """Given a NetworkX node, and a list of ArangoDB vertex collections defined,
+        identify which ArangoDB vertex collection it should belong to.
 
-        NOTE: You must override this function if your NetworkX graph is NOT Homogeneous
-        or does NOT comply to ArangoDB standards (i.e the node IDs are not formatted
-        like "{collection}/{key}").
+        NOTE: You must override this function if len(**adb_v_cols**) > 1
+        OR **nx_node_id* does NOT comply to ArangoDB standards
+        (i.e "{collection}/{key}").
 
         :param nx_node_id: The NetworkX ID of the node.
         :type nx_node_id: adbnx_adapter.typings.NxId
         :param nx_node: The NetworkX node object.
         :type nx_node: adbnx_adapter.typings.NxData
+        :param adb_v_cols: All ArangoDB vertex collections specified
+            by the **edge_definitions** parameter of networkx_to_arangodb()
+        :type adb_v_cols: List[str]
         :return: The ArangoDB collection name
         :rtype: str
         """
-        # In this case, nx_node_id is already a valid ArangoDB _id
+        # In this case, we assume that **nx_node_id** is already a valid ArangoDB _id
+        # Otherwise, user must override this function
         adb_vertex_id: str = str(nx_node_id)
         return adb_vertex_id.split("/")[0]
 
@@ -77,17 +76,19 @@ class ADBNX_Controller(Abstract_ADBNX_Controller):
         nx_edge: NxData,
         from_nx_node: NxData,
         to_nx_node: NxData,
+        adb_e_cols: List[str],
     ) -> str:
-        """Given a NetworkX edge, and its pair of nodes, identify what ArangoDB
-        collection should it belong to.
+        """Given a NetworkX edge, its pair of nodes, and a list of ArangoDB
+        edge collections defined, identify which ArangoDB edge collection it
+        should belong to.
 
-        NOTE #1: You must override this function if your NetworkX graph is NOT
-        Homogeneous or does NOT comply to ArangoDB standards
-        (i.e the edge IDs are not formatted like "{collection}/{key}").
+        NOTE #1: You must override this function if len(**adb_e_cols**) > 1
+        OR **nx_edge["_id"]** does NOT comply to ArangoDB standards
+        (i.e "{collection}/{key}").
 
-        NOTE #2: You can accesss the ID & Collection belonging to the
-        **from_nx_node** & **to_nx_node** parameters via their "nx_id" & "col"
-        attribute keys. E.g `from_collection = from_nx_node["col"]`
+        NOTE #2: The two nodes associated to the **nx_edge** can be accessed
+        by the **from_nx_node** & **to_nx_node** parameters, and are guaranteed
+        to have the following attributes: `{"nx_id", "adb_id", "adb_col", "adb_key"}`
 
         :param nx_edge: The NetworkX edge object.
         :type nx_edge: adbnx_adapter.typings.NxData
@@ -95,10 +96,15 @@ class ADBNX_Controller(Abstract_ADBNX_Controller):
         :type from_nx_node: adbnx_adapter.typings.NxData
         :param to_nx_node: The NetworkX node object representing the edge destination.
         :type to_nx_node: adbnx_adapter.typings.NxData
+        :param adb_e_cols: All ArangoDB edge collections specified
+            by the **edge_definitions** parameter of
+            ADBNX_Adapter.networkx_to_arangodb()
+        :type adb_e_cols: List[str]
         :return: The ArangoDB collection name
         :rtype: str
         """
-        # In this case, nx_edge["_id"] is already a valid ArangoDB _id
+        # In this case, we assume that nx_edge["_id"] is already a valid ArangoDB _id
+        # Otherwise, user must override this function
         adb_edge_id: str = nx_edge["_id"]
         return adb_edge_id.split("/")[0]
 
@@ -106,10 +112,8 @@ class ADBNX_Controller(Abstract_ADBNX_Controller):
         """Given a NetworkX node, derive its valid ArangoDB key.
 
         NOTE: You must override this function if you want to create custom ArangoDB _key
-        values for your NetworkX nodes or if your NetworkX graph does NOT comply to
-        ArangoDB standards (i.e the node IDs are not formatted
-        like "{collection}/{key}"). For more  info, see the **keyify_nodes**
-        parameter of ADBNX_Adapter.networkx_to_arangodb()
+        values from your NetworkX nodes. To enable the use of this method, enable the
+        **keyify_nodes** parameter in ADBNX_Adapter.networkx_to_arangodb().
 
         :param nx_node_id: The NetworkX node id.
         :type nx_node_id: adbnx_adapter.typings.NxId
@@ -120,7 +124,8 @@ class ADBNX_Controller(Abstract_ADBNX_Controller):
         :return: A valid ArangoDB _key value.
         :rtype: str
         """
-        # In this case, nx_node_id is already a valid ArangoDB _id
+        # In this case, we assume that **nx_node_id** is already a valid ArangoDB _id
+        # Otherwise, user must override this function
         adb_vertex_id: str = str(nx_node_id)
         return adb_vertex_id.split("/")[1]
 
@@ -135,14 +140,12 @@ class ADBNX_Controller(Abstract_ADBNX_Controller):
         its valid ArangoDB key.
 
         NOTE #1: You must override this function if you want to create custom ArangoDB
-        _key values for your NetworkX edges or if your NetworkX graph does NOT comply
-        to ArangoDB standards (i.e the edge IDs are not formatted
-        like "{collection}/{key}"). For more info, see the **keyify_edges**
-        parameter of ADBNX_Adapter.networkx_to_arangodb()
+        _key values from your NetworkX edges. To enable the use of this method, enable
+        the **keyify_edges** parameter in ADBNX_Adapter.networkx_to_arangodb().
 
-        NOTE #2: You can accesss the ID & Collection belonging to the
-        **from_nx_node** & **to_nx_node** parameters via their "nx_id" & "col"
-        attribute keys. E.g `from_collection = from_nx_node["col"]`
+        NOTE #2: The two nodes associated to the **nx_edge** can be accessed
+        by the **from_nx_node** & **to_nx_node** parameters, and are guaranteed
+        to have the following attributes: `{"nx_id", "adb_id", "adb_col", "adb_key"}`
 
         :param nx_edge: The NetworkX edge object.
         :type nx_edge: adbnx_adapter.typings.NxData
@@ -155,7 +158,8 @@ class ADBNX_Controller(Abstract_ADBNX_Controller):
         :return: A valid ArangoDB _key value.
         :rtype: str
         """
-        # In this case, nx_edge["_id"] is already a valid ArangoDB _id
+        # In this case, we assume that nx_edge["_id"] is already a valid ArangoDB _id
+        # Otherwise, user must override this function
         adb_edge_id: str = nx_edge["_id"]
         return adb_edge_id.split("/")[1]
 
